//+--------------------------------------------------------------------------------------+
//|                                                                                      |
//+--------------------------------------------------------------------------------------+
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1  clrBlue//DeepSkyBlue
#property indicator_color2  clrRed//PaleVioletRed
#property indicator_color3  clrRed//PaleVioletRed
#property indicator_color4  clrBlue//DodgerBlue
#property indicator_color5  clrRed//Magenta
#property indicator_width1  3
#property indicator_width2  3
#property indicator_width3  3
#property indicator_minimum 0
#property indicator_maximum 0.5

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased   // Heiken ashi trend biased price
};

//
//
//
//
//

#import "libSSA.dll"
   void fastSingular(double& sourceArray[],int arraySize, int lag, int numberOfComputationLoops, double& destinationArray[]);
#import

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame      = PERIOD_CURRENT;
extern enPrices        SSAPrice       = pr_close; // Price to use 
extern int    SSALag                  = 25;
extern int    SSANumberOfComputations = 2;
extern int    SSANumberOfBars         = 300;
extern int    FirstBar                = 300; 
extern bool   alertsOn                = true;
extern bool   alertsOnCurrent         = false;
extern bool   alertsMessage           = true;
extern bool   alertsSound             = false;
extern bool   alertsNotify            = false;
extern bool   alertsEmail             = false;
extern string soundFile               = "alert2.wav";
extern bool   Interpolate             = true;

extern bool   MultiColor              = true;
extern bool   ShowArrows              = true;
extern int    arrowSize               = 2;
extern bool   ArrowsOnFirstBar        = true;


//
//
//
//
//

double in[];
double ssaDa[];
double ssaDb[];
double arrowUp[];
double arrowDn[];
double pr[];
double slope[];
double ssaIn[];
double ssaOut[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;

//+--------------------------------------------------------------------------------------+
//|                                                                                      |
//+--------------------------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
      SetIndexBuffer(0,in);
      SetIndexBuffer(1,ssaDa);
      SetIndexBuffer(2,ssaDb);
      SetIndexBuffer(3,arrowUp);   
      SetIndexBuffer(4,arrowDn);  
      SetIndexBuffer(5,pr);
      SetIndexBuffer(6,slope);
      
      if (ShowArrows)
      {
        SetIndexStyle(3,DRAW_ARROW,0,arrowSize); SetIndexArrow(3,233);
        SetIndexStyle(4,DRAW_ARROW,0,arrowSize); SetIndexArrow(4,234);   }
      else
      {
        SetIndexStyle(3,DRAW_NONE);
        SetIndexStyle(4,DRAW_NONE);
      }       
      
         indicatorFileName = WindowExpertName();
         returnBars        = TimeFrame==-99;
         TimeFrame         = MathMax(TimeFrame,_Period);
   IndicatorShortName(timeFrameToString(TimeFrame)+" SSA normalized end-pointed");
   return(0);
}
int deinit(){return(0);}

//+--------------------------------------------------------------------------------------+
//|                                                                                      |
//+--------------------------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,limit,counted_bars = IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
      limit = MathMin(Bars-counted_bars,Bars-1);
      if (returnBars) { in[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
      
   if (TimeFrame==Period())
   {
     if (MultiColor && slope[limit]==-1) CleanPoint(limit,ssaDa,ssaDb);
     for(i=limit; i>=0; i--)
     {
        pr[i] = getPrice(SSAPrice,Open,Close,High,Low,i);

        //
        //
        //
        //
        //
      
        if (i<=FirstBar)
        {
           int ssaBars = MathMin(Bars-i,SSANumberOfBars);
           if (ssaBars<SSALag) continue;
              if (ArraySize(ssaIn) != ssaBars)
              {
                 ArrayResize(ssaIn ,ssaBars);
                 ArrayResize(ssaOut,ssaBars);
              }
              ArrayCopy(ssaIn,pr,0,i,ssaBars);
      
          fastSingular(ssaIn,ssaBars,SSALag,SSANumberOfComputations,ssaOut);
          in[i]=ssaOut[0];
          
          ssaDa[i] = EMPTY_VALUE;
          ssaDb[i] = EMPTY_VALUE;
          slope[i] = slope[i+1];

          if (in[i]<in[i+1]) slope[i] = -1;
          if (in[i]>in[i+1]) slope[i] =  1;
          if (MultiColor && slope[i] == -1) PlotPoint(i,ssaDa,ssaDb,in);
          
          //
          //
          //
          //
          //
      
          arrowUp[i] = EMPTY_VALUE;
          arrowDn[i] = EMPTY_VALUE;
          if (slope[i]!= slope[i+1])
          //if ( i>0 && slope[i]!= slope[i+1])
          if (slope[i] == 1)
                arrowUp[i] = in[i] - iATR(NULL,0,20,i)/2.0;
          else  arrowDn[i] = in[i] + iATR(NULL,0,20,i)/2.0;
       }                   
   }
   
   //
   //
   //
   //
   //
      
   if (alertsOn)
   {
     if (alertsOnCurrent)
          int whichBar = 0;
     else     whichBar = 1;
     if (slope[whichBar] != slope[whichBar+1])
     if (slope[whichBar] == 1)
           doAlert("sloping up");
     else  doAlert("sloping down");       
   }        
   SetIndexDrawBegin(0,Bars-FirstBar);  
   SetIndexDrawBegin(1,Bars-FirstBar);
   SetIndexDrawBegin(2,Bars-FirstBar);                  
   return(0); 
   }
   
   //
   //
   //
   //
   //
   
   int shift = -1; if (ArrowsOnFirstBar) shift=1;
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   if (MultiColor && slope[limit]==-1) CleanPoint(limit,ssaDa,ssaDb);
   for(i=limit; i>=0; i--) 
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
      int x = iBarShift(NULL,TimeFrame,Time[i+shift]);
         in[i]      = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,SSAPrice,SSALag,SSANumberOfComputations,SSANumberOfBars,FirstBar,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,0,y);
         ssaDa[i]   = EMPTY_VALUE;
         ssaDb[i]   = EMPTY_VALUE;
         slope[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,SSAPrice,SSALag,SSANumberOfComputations,SSANumberOfBars,FirstBar,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,6,y);
      if(x!=y)
      {
         arrowUp[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,SSAPrice,SSALag,SSANumberOfComputations,SSANumberOfBars,FirstBar,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,3,y);
         arrowDn[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,SSAPrice,SSALag,SSANumberOfComputations,SSANumberOfBars,FirstBar,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,4,y);
      }
      else
      {
         arrowUp[i] = EMPTY_VALUE;
         arrowDn[i] = EMPTY_VALUE;
      }
         //
         //
         //
         //
         //
                        
         if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,TimeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            for(int k = 1; k < n; k++) in[i+k] = in[i] + (in[i+n] - in[i]) * k/n;          
   }
   SetIndexDrawBegin(0,Bars-FirstBar*TimeFrame/Period());
   SetIndexDrawBegin(1,Bars-FirstBar*TimeFrame/Period());
   SetIndexDrawBegin(2,Bars-FirstBar*TimeFrame/Period());
   for(i=limit; i>=0; i--) if (MultiColor && slope[i] == -1) PlotPoint(i,ssaDa,ssaDb,in); 
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," SSA end-pointed ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," SSA end-pointed "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//
//

double workHa[][4];
double getPrice(int price, const double& open[], const double& close[], const double& high[], const double& low[], int i, int instanceNo=0)
{
  if (price>=pr_haclose && price<=pr_hatbiased)
   {
      if (ArrayRange(workHa,0)!= Bars) ArrayResize(workHa,Bars);
         int r = Bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen;
         if (r>0)
                haOpen  = (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0;
         else   haOpen  = (open[i]+close[i])/2;
         double haClose = (open[i] + high[i] + low[i] + close[i]) / 4.0;
         double haHigh  = MathMax(high[i], MathMax(haOpen,haClose));
         double haLow   = MathMin(low[i] , MathMin(haOpen,haClose));

         if(haOpen  <haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else                 { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                                workHa[r][instanceNo+2] = haOpen;
                                workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (price)
         {
            case pr_haclose:     return(haClose);
            case pr_haopen:      return(haOpen);
            case pr_hahigh:      return(haHigh);
            case pr_halow:       return(haLow);
            case pr_hamedian:    return((haHigh+haLow)/2.0);
            case pr_hamedianb:   return((haOpen+haClose)/2.0);
            case pr_hatypical:   return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:  return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:   return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (price)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
   }
   return(0);
}

