//+------------------------------------------------------------------+
//|                                        Slope_MA_Confirmation.mq4 |
//|                             Copyright (c) 2016, Gehtsoft USA LLC | 
//|                                            http://fxcodebase.com |
//|                                   Paypal: https://goo.gl/9Rj74e  | 
//+------------------------------------------------------------------+
//|                                      Developed by : Mario Jemic  |                    
//|                                          mario.jemic@gmail.com   |
//|                   BitCoin : 15VCJTLaz12Amr7adHSBtL9v8XomURo9RF   |
//+------------------------------------------------------------------+


#property indicator_buffers 12
#property indicator_chart_window

/* Candles */
#property indicator_color1 clrLime
#property indicator_color2 clrLime
#property indicator_color3 clrLime
#property indicator_color4 clrLime
#property indicator_color5 clrRed
#property indicator_color6 clrRed
#property indicator_color7 clrRed
#property indicator_color8 clrRed
#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 3
#property indicator_width4 3
#property indicator_width5 1
#property indicator_width6 1
#property indicator_width7 3
#property indicator_width8 3

/* Slope */
#property indicator_color9 clrYellow
#property indicator_style9 STYLE_SOLID
#property indicator_width9 2
#property indicator_color10 clrLime
#property indicator_style10 STYLE_SOLID
#property indicator_width10 2
#property indicator_color11 clrRed
#property indicator_style11 STYLE_SOLID
#property indicator_width11 2

/* MA */
#property indicator_color12 clrMagenta
#property indicator_style12 STYLE_SOLID
#property indicator_width12 2

/* Parameters */
enum e_method{ SMA=MODE_SMA, EMA=MODE_EMA, SMMA=MODE_SMMA, LWMA=MODE_LWMA };
enum e_price{ CLOSE=PRICE_CLOSE, OPEN=PRICE_OPEN, LOW=PRICE_LOW, HIGH=PRICE_HIGH, MEDIAN=PRICE_MEDIAN, TYPICAL=PRICE_TYPICAL, WEIGHTED=PRICE_WEIGHTED };

extern string   Comment0         = "- MA Values -";
extern int      MA_Period        = 50;
extern e_method MA_Method        = SMA;
extern e_price  MA_Price_Type    = CLOSE;
extern string   Comment1         = "- Slope Values -";
extern int      Slope_Period     = 50;
extern e_method Slope_Method     = SMA;
extern e_price  Slope_Price_Type = CLOSE;
extern int      Limit_Bars       = 800;

// Candles
double Up_High[];
double Up_Low[];
double Up_Open[];
double Up_Close[];
double Dn_High[];
double Dn_Low[];
double Dn_Open[];
double Dn_Close[];

// Slope
double Trend[];
double Up[];
double Dn[];
double MA1[];
double MA2[];
double Vect[];

// MA
double MA[];

int Bullish = 1;
int Bearish = -1;
int Neutral = 0;

int init(){
   
   IndicatorShortName("Slope/MA Price Overlay");
   IndicatorBuffers(15);
   
   SetIndexBuffer(0,Up_High);
   SetIndexBuffer(1,Up_Low);
   SetIndexBuffer(2,Up_Open);
   SetIndexBuffer(3,Up_Close);
   SetIndexBuffer(4,Dn_High);
   SetIndexBuffer(5,Dn_Low);
   SetIndexBuffer(6,Dn_Open);
   SetIndexBuffer(7,Dn_Close);
   
   for (int i=0; i<8; i++){
      SetIndexStyle(i,DRAW_HISTOGRAM);
   }
   
   SetIndexStyle(8,DRAW_LINE);
   SetIndexBuffer(8,Trend);
   SetIndexLabel(8,"Slope Trend");
   
   SetIndexStyle(9,DRAW_LINE);
   SetIndexBuffer(9,Up);
   SetIndexLabel(9,"Up Trend");
   
   SetIndexStyle(10,DRAW_LINE);
   SetIndexBuffer(10,Dn);
   SetIndexLabel(10,"Dn Trend");
   
   SetIndexBuffer(11,MA1);
   SetIndexBuffer(12,MA2);
   SetIndexBuffer(13,Vect);
   
   SetIndexStyle(14,DRAW_LINE);
   SetIndexBuffer(14,MA);
   SetIndexLabel(14,"MA");
   
   return(0);
}

int start()
  {
   
   int i, bias;
   int counted_bars=IndicatorCounted();
   int limit = Bars-counted_bars-1;
   
   if (Limit_Bars > 0) limit = Limit_Bars;
   
   for(i=limit; i>=0; i--){
   
      MA [i]  = iMA(NULL,0,MA_Period,0,ENUM_MA_METHOD(MA_Method),ENUM_APPLIED_PRICE(MA_Price_Type),i);
      MA1[i]  = iMA(NULL,0,Slope_Period,0,ENUM_MA_METHOD(Slope_Method),ENUM_APPLIED_PRICE(Slope_Price_Type),i);
      MA2[i]  = iMA(NULL,0,MathFloor(Slope_Period/2),0,ENUM_MA_METHOD(Slope_Method),ENUM_APPLIED_PRICE(Slope_Price_Type),i);
      Vect[i] = 2*MA2[i]-MA1[i];
      Trend[i] = iMAOnArray(Vect,0,MathFloor(MathSqrt(Slope_Period)),0,ENUM_MA_METHOD(Slope_Method),i);
      if (Trend[i] > Trend[i+1]){
         Up[i] = Trend[i];
         Dn[i] = EMPTY_VALUE;
      }
      
      if (Trend[i] < Trend[i+1]){
         Dn[i] = Trend[i];
         Up[i] = EMPTY_VALUE;
      }
      
      bias=ColorCandle(Trend[i], Trend[i+1], MA[i], MA[i+1]);
   
      if(bias>0){
         
         Up_High[i]  = iHigh(NULL,0,i);
         Up_Low[i]   = iLow(NULL,0,i);  
         Up_Open[i]  = iOpen(NULL,0,i);
         Up_Close[i] = iClose(NULL,0,i);  
      
      }
      else if(bias<0){
         
         Dn_High[i]  = iHigh(NULL,0,i);
         Dn_Low[i]   = iLow(NULL,0,i);  
         Dn_Open[i]  = iOpen(NULL,0,i);
         Dn_Close[i] = iClose(NULL,0,i);
      
      }
      
   }
   
//----
   return(0);
}

int ColorCandle(double current_slope, double previous_slope, double current_ma, double previous_ma){
 
 if (current_slope > previous_slope && current_ma > previous_ma)
   
   return (Bullish);
 
 else if (current_slope < previous_slope && current_ma < previous_ma)
   
   return (Bearish);
 
 else
   
   return (Neutral);

}