#property indicator_separate_window
#property indicator_buffers    3
#property indicator_color1     clrLimeGreen
#property indicator_color2     clrPaleVioletRed
#property indicator_color3     clrPaleVioletRed
#property indicator_width3     2
#property indicator_style1     STYLE_DOT
#property indicator_style2     STYLE_DOT
#property indicator_level1     0
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor clrDarkSlateGray
#property strict

//
//
//

extern ENUM_TIMEFRAMES   TimeFrame       = PERIOD_CURRENT;    // Time frame
input int                FastPeriod      = 10;
input ENUM_MA_METHOD     FastMethod      = MODE_EMA;
input ENUM_APPLIED_PRICE FastPrice       = PRICE_CLOSE;
input int                SlowPeriod      = 50;
input ENUM_MA_METHOD     SlowMethod      = MODE_EMA;
input ENUM_APPLIED_PRICE SlowPrice       = PRICE_CLOSE;
input ENUM_APPLIED_PRICE Price           = PRICE_CLOSE;
input int                PricePeriod     = 5;
input ENUM_MA_METHOD     PriceMethod     = MODE_EMA;
input bool               alertsOn        = true;              // Turn alerts on?
input bool               alertsOnCurrent = false;             // Alerts on still opened bar?
input bool               alertsMessage   = true;              // Alerts should show popup message?
input bool               alertsSound     = false;             // Alerts should play a sound?
input bool               alertsEmail     = false;             // Alerts should send email?
input bool               alertsNotify    = false;             // Alerts should send notification?
input bool               Interpolate     = true;

double bandUp[],bandDn[],price[],trend[],count[];
string indicatorFileName;
#define _mtfCall(_buff,_ind) iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,FastPeriod,FastMethod,FastPrice,SlowPeriod,SlowMethod,SlowPrice,Price,PricePeriod,PriceMethod,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,alertsNotify,_buff,_ind)

//------------------------------------------------------------------
//
//------------------------------------------------------------------

int OnInit()
{
   IndicatorDigits(6); 
   IndicatorBuffers(5);
   SetIndexBuffer(0,bandUp);
   SetIndexBuffer(1,bandDn);
   SetIndexBuffer(2,price);
   SetIndexBuffer(3,trend);
   SetIndexBuffer(4,count);
   
   indicatorFileName = WindowExpertName();
   TimeFrame         = fmax(TimeFrame,_Period); 
   
   IndicatorSetString(INDICATOR_SHORTNAME,timeFrameToString(TimeFrame)+" price between 2 ma's ("+(string)FastPeriod+","+(string)SlowPeriod+","+(string)PricePeriod+",)");      
return(INIT_SUCCEEDED);  
}  
int deinit() { return(0);  }  

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   int i,limit=fmin(rates_total-prev_calculated+1,rates_total-1); count[0] = limit;
      if (TimeFrame!=_Period)
      {
         limit = (int)fmax(limit,fmin(rates_total-1,_mtfCall(4,0)*TimeFrame/_Period));
         for(i=limit; i>=0; i--)
         {
             int y = iBarShift(NULL,TimeFrame,time[i]);
                bandUp[i] = _mtfCall(0,y);
                bandDn[i] = _mtfCall(1,y);
                price[i]  = _mtfCall(2,y);
                
                //
                //
                //
                     
                if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,time[i-1]))) continue;
                  #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
                  int n,k; datetime btime = iTime(NULL,TimeFrame,y);
                     for(n = 1; (i+n)<rates_total && time[i+n] >= btime; n++) continue;	
                     for(k = 1; k<n && (i+n)<rates_total && (i+k)<rates_total; k++) 
                     {
                       _interpolate(bandUp); 
                       _interpolate(bandDn);
                       _interpolate(price);
                     }
          }                                                                              
   
   return(rates_total);
   }

   //
   //
   //
   
   for(i=limit; i>=0; i--)
   {
      double ma1 = iMA(NULL,0,FastPeriod ,0,FastMethod ,FastPrice,i);
      double ma2 = iMA(NULL,0,SlowPeriod ,0,SlowMethod ,SlowPrice,i);
      double prc = iMA(NULL,0,PricePeriod,0,PriceMethod,    Price,i);
         double range = fabs(ma1-ma2);
         while (true)
         {
            if (prc>ma1 && prc>ma2) { price[i] =  1;              break; }
            if (prc<ma1 && prc<ma2) { price[i] = -1;              break; }
            if (range!=0)           { price[i] = (prc-ma2)/range; break; }
                                      price[i] = 0;               break;
         }            
      bandUp[i] =  1;
      bandDn[i] = -1;
      trend[i]  = (i<rates_total-1) ? (price[i]>0) ? 1 : (price[i]<0) ? -1 : trend[i+1] : 0;
   }
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"breaking up");
         if (trend[whichBar] ==-1) doAlert(whichBar,"breaking down");
      }         
   }
return(rates_total);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," price between 2 mas ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," price between 2 mas "),message);
          if (alertsNotify)  SendNotification(message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}
