//+------------------------------------------------------------------+
//|                                                                  |
//| idea for zero lag hull moving average                            |
//| originaly by spikoloco                                           }
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1  clrDeepSkyBlue
#property indicator_color2  clrRed
#property indicator_color3  clrYellow
#property strict

//
//
//
//
//

enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_tbiased2,   // Trend biased (extreme) price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased,  // Heiken ashi trend biased price
   pr_hatbiased2, // Heiken ashi trend biased (extreme) price
   pr_habclose,   // Heiken ashi (better formula) close
   pr_habopen ,   // Heiken ashi (better formula) open
   pr_habhigh,    // Heiken ashi (better formula) high
   pr_hablow,     // Heiken ashi (better formula) low
   pr_habmedian,  // Heiken ashi (better formula) median
   pr_habtypical, // Heiken ashi (better formula) typical
   pr_habweighted,// Heiken ashi (better formula) weighted
   pr_habaverage, // Heiken ashi (better formula) average
   pr_habmedianb, // Heiken ashi (better formula) median body
   pr_habtbiased, // Heiken ashi (better formula) trend biased price
   pr_habtbiased2 // Heiken ashi (better formula) trend biased (extreme) price
};

#define _disLin 1
#define _disDot 4

enum enDisplay
{
   en_lin = _disLin,                  // Display line
   en_lid = _disLin+_disDot,          // Display lines with dots
   en_dot = _disDot                   // Display dots
};

extern ENUM_TIMEFRAMES TimeFrame       = PERIOD_CURRENT;   // Time frame
extern int             Length          = 20;               // Zerolag Hull length
extern enPrices        Price           = pr_close;         // Price to use 
extern enDisplay       DisplayType     = en_lin;           // Display type
extern int             LinesWidth      = 3;                // Lines width (when lines are included in display)
extern bool            ArrowOnFirst    = true;             // Arrow on first bars
extern int             UpArrowSize     = 2;                // Up Arrow size
extern int             DnArrowSize     = 2;                // Down Arrow size
extern int             UpArrowCode     = 159;              // Up Arrow code
extern int             DnArrowCode     = 159;              // Down arrow code
extern double          UpArrowGap      = 0.5;              // Up Arrow gap        
extern double          DnArrowGap      = 0.5;              // Dn Arrow gap
extern color           UpArrowColor    = clrLimeGreen;     // Up Arrow Color
extern color           DnArrowColor    = clrOrange;        // Down Arrow Color
extern bool            Interpolate     = true;             // Interpolate true/false

double val[],hmada[],hmadb[],arrowu[],arrowd[],trend[],count[];
string indicatorFileName;
#define _mtfCall(_buff,_ind) iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Length,Price,DisplayType,LinesWidth,ArrowOnFirst,UpArrowSize,DnArrowSize,UpArrowCode,DnArrowCode,UpArrowGap,DnArrowGap,UpArrowColor,DnArrowColor,0,_buff,_ind)

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int OnInit()
{
    int lstyle = DRAW_LINE;  if ((DisplayType&_disLin)==0) lstyle = DRAW_NONE;
    int astyle = DRAW_ARROW; if ((DisplayType&_disDot)==0) astyle = DRAW_NONE;
    IndicatorBuffers(7);
    SetIndexBuffer(0,val);     SetIndexStyle(0,lstyle,EMPTY,LinesWidth);
    SetIndexBuffer(1,hmada);   SetIndexStyle(1,lstyle,EMPTY,LinesWidth);
    SetIndexBuffer(2,hmadb);   SetIndexStyle(2,lstyle,EMPTY,LinesWidth);
    SetIndexBuffer(3,arrowu);  SetIndexStyle(3,astyle,0,UpArrowSize,UpArrowColor); SetIndexArrow(3,UpArrowCode);
    SetIndexBuffer(4,arrowd);  SetIndexStyle(4,astyle,0,DnArrowSize,DnArrowColor); SetIndexArrow(4,DnArrowCode);
    SetIndexBuffer(5,trend); 
    SetIndexBuffer(6,count); 

    indicatorFileName = WindowExpertName();
    TimeFrame         = fmax(TimeFrame,_Period);
         
    IndicatorShortName(timeFrameToString(TimeFrame)+" zero lag HMA ("+(string)Length+")");         
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason) { }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
    int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = fmin(Bars-counted_bars,Bars-1); count[0] = limit;
         if (TimeFrame != _Period)
         {
            limit = (int)fmax(limit,fmin(Bars-1,_mtfCall(6,0)*TimeFrame/_Period));
            if (trend[limit] == -1) CleanPoint(limit,hmada,hmadb);
            for (i=limit;i>=0 && !_StopFlag; i--)
            {
                int y = iBarShift(NULL,TimeFrame,Time[i]);
                int x = y;
                if (ArrowOnFirst)
                      {  if (i<Bars-1) x = iBarShift(NULL,TimeFrame,Time[i+1]);               }
                else  {  if (i>0)      x = iBarShift(NULL,TimeFrame,Time[i-1]); else x = -1;  }
                     val[i]    = _mtfCall(0,y);
   	               hmada[i]  = EMPTY_VALUE;
                     hmadb[i]  = EMPTY_VALUE;
                     arrowu[i] = EMPTY_VALUE;
                     arrowd[i] = EMPTY_VALUE;
                     trend[i]  = _mtfCall(5,y);
                     if (x!=y)
                     {
                       arrowu[i] = _mtfCall(3,y);
                       arrowd[i] = _mtfCall(4,y);
                     }
                     
                     //
                     //
                     //
                     //
                     //
                     
                     if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;
                     #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
                     int n,k; datetime time = iTime(NULL,TimeFrame,y);
                        for(n = 1; (i+n)<Bars && Time[i+n] >= time; n++) continue;	
                        for(k = 1; k<n && (i+n)<Bars && (i+k)<Bars; k++) _interpolate(val);                       
            }
            for (i=limit; i >= 0; i--) if (trend[i] == -1) PlotPoint(i,hmada,hmadb,val); 
      return(0);
      }
      
      //
      //
      //
      //
      //
      
      if (trend[limit] == -1) CleanPoint(limit,hmada,hmadb);
      for(i=limit; i>=0; i--)
      {
         double hma1 = iHma(getPrice(Price,Open,Close,High,Low,i,Bars),Length,i,Bars,0);
         double hma2 = iHma(hma1                                      ,Length,i,Bars,1);
         val[i] = 2.0*hma1-hma2;

         //
         //
         //
         //
         //
         
         hmada[i]  = EMPTY_VALUE;
         hmadb[i]  = EMPTY_VALUE;
         arrowu[i] = EMPTY_VALUE;
         arrowd[i] = EMPTY_VALUE;
         trend[i] = (i<Bars-1) ? (val[i]>val[i+1])  ? 1 : (val[i]<val[i+1]) ? -1 : trend[i+1] : 0;
         if (trend[i] == -1) PlotPoint(i,hmada,hmadb,val);
         if (i<Bars-1 && trend[i]!=trend[i+1])
         {
            if (trend[i] ==  1) arrowu[i] = fmin(val[i],Low[i] )-iATR(NULL,0,15,i)*UpArrowGap;
            if (trend[i] == -1) arrowd[i] = fmax(val[i],High[i])+iATR(NULL,0,15,i)*DnArrowGap;
         }
      }
return(0);
}      

//+----------------------------------------------------------------------------------+
//|                                                                                  |
//+----------------------------------------------------------------------------------+
//
//
//
//
//

#define _hullInstances 2
#define _hullInstancesSize 2
double workHull[][_hullInstances*_hullInstancesSize];

double iHma(double price,double period,int r,int bars,int instanceNo=0)
{
      if(ArrayRange(workHull,0)!=bars) ArrayResize(workHull,bars);  r = bars-r-1; 
      instanceNo*=_hullInstancesSize; workHull[r][instanceNo]=price;
      if(period<=1) return(price);
   
      //
      //
      //
      //
      //
   
      int HmaPeriod  = (int)fmax(period,2);
      int HalfPeriod = (int)floor(HmaPeriod/2);
      int HullPeriod = (int)floor(sqrt(HmaPeriod));
      
      //
      //
      //
      //
      //
      
      double hma,hmw,weight;
      hmw=HalfPeriod; hma=hmw*price;
      for(int k=1; k<HalfPeriod && (r-k)>=0; k++)
      {
         weight = HalfPeriod-k;
         hmw   += weight;
         hma   += weight*workHull[r-k][instanceNo];
      }
      workHull[r][instanceNo+1]=2.0*hma/hmw;
      
      //
      //
      //
      //
      //
      
      hmw=HmaPeriod; hma=hmw*price;
      for(int k=1; k<period && (r-k)>=0; k++)
      {
         weight = HmaPeriod-k;
         hmw   += weight;
         hma   += weight*workHull[r-k][instanceNo];
      }
      workHull[r][instanceNo+1]-=hma/hmw;
      
      //
      //
      //
      //
      //
      
      hmw=HullPeriod; hma=hmw*workHull[r][instanceNo+1];
      for(int k=1; k<HullPeriod && (r-k)>=0; k++)
      {
         weight = HullPeriod-k;
         hmw   += weight;
         hma   += weight*workHull[r-k][1+instanceNo];
      }
return(hma/hmw);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

#define _prHABF(_prtype) (_prtype>=pr_habclose && _prtype<=pr_habtbiased2)
#define _priceInstances     1
#define _priceInstancesSize 4
double workHa[][_priceInstances*_priceInstancesSize];
double getPrice(int tprice, const double& open[], const double& close[], const double& high[], const double& low[], int i, int bars, int instanceNo=0)
{
  if (tprice>=pr_haclose)
   {
      if (ArrayRange(workHa,0)!= bars) ArrayResize(workHa,bars); instanceNo*=_priceInstancesSize; int r = bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen  = (r>0) ? (workHa[r-1][instanceNo+2] + workHa[r-1][instanceNo+3])/2.0 : (open[i]+close[i])/2;;
         double haClose = (open[i]+high[i]+low[i]+close[i]) / 4.0;
         if (_prHABF(tprice))
               if (high[i]!=low[i])
                     haClose = (open[i]+close[i])/2.0+(((close[i]-open[i])/(high[i]-low[i]))*MathAbs((close[i]-open[i])/2.0));
               else  haClose = (open[i]+close[i])/2.0; 
         double haHigh  = fmax(high[i], fmax(haOpen,haClose));
         double haLow   = fmin(low[i] , fmin(haOpen,haClose));

         //
         //
         //
         //
         //
         
         if(haOpen<haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else               { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                              workHa[r][instanceNo+2] = haOpen;
                              workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (tprice)
         {
            case pr_haclose:
            case pr_habclose:    return(haClose);
            case pr_haopen:   
            case pr_habopen:     return(haOpen);
            case pr_hahigh: 
            case pr_habhigh:     return(haHigh);
            case pr_halow:    
            case pr_hablow:      return(haLow);
            case pr_hamedian:
            case pr_habmedian:   return((haHigh+haLow)/2.0);
            case pr_hamedianb:
            case pr_habmedianb:  return((haOpen+haClose)/2.0);
            case pr_hatypical:
            case pr_habtypical:  return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:
            case pr_habweighted: return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:  
            case pr_habaverage:  return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
            case pr_habtbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
            case pr_hatbiased2:
            case pr_habtbiased2:
               if (haClose>haOpen)  return(haHigh);
               if (haClose<haOpen)  return(haLow);
                                    return(haClose);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (tprice)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
      case pr_tbiased2:   
               if (close[i]>open[i]) return(high[i]);
               if (close[i]<open[i]) return(low[i]);
                                     return(close[i]);        
   }
return(0);
}

//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}
 
//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}


